/* ---------------------------------------------------------- */
/* Originated from Gian-Carlo Pascutto (foobar2000 convolver) */
/* Copyright (C) 2003  http://sjeng.org                       */
/* ---------------------------------------------------------- */

#include <math.h>
#include "defs.h"
#include "fatfs.h"

#include "convolver.h"

//#define FFT_LEN             65536 * 2
//#define HALF_FFT_LEN        65536

// global variables
static float scale_coef, scale_coef2;

static float *lobuf = (float *)0x02000000; // lobuf = (float *)calloc(HALF_FFT_LEN, sizeof(float)); -> 0x40000
static float *robuf = (float *)0x02040000; // robuf = (float *)calloc(HALF_FFT_LEN, sizeof(float)); -> 0x40000

static float *fftL = (float *)0x02100000;  // fftL = (float *)calloc(2*FFT_LEN, sizeof(float)); -> 0x100000
static float *fftR = (float *)0x02200000;  // fftR = (float *)calloc(2*FFT_LEN, sizeof(float)); -> 0x100000
static float *fftLout = (float *)0x02300000; // fftLout = (float *)calloc(2*FFT_LEN, sizeof(float)); -> 0x100000
static float *fftRout = (float *)0x02400000; // fftRout = (float *)calloc(2*FFT_LEN, sizeof(float)); -> 0x100000

static float *impulseDataLeft = (float *)0x02500000; // impulseDataLeft = (float *)calloc(FFT_LEN, sizeof(float)); -> 0x80000
static float *impulseDataRight = (float *)0x02580000;  // impulseDataRight = (float *)calloc(FFT_LEN, sizeof(float)); -> 0x80000

static float *responseLeft = (float *)0x02600000;  // responseLeft = (float *)calloc(2*FFT_LEN, sizeof(float)); -> 0x100000
static float *responseRight = (float *)0x02700000; // responseRight = (float *)calloc(2*FFT_LEN, sizeof(float)); -> 0x100000

void convolverClearOutputBuffers() {
  unsigned int i;

  for (i = 0; i < HALF_FFT_LEN; i++) {
    lobuf[i] = 0.0;
    robuf[i] = 0.0;
  }
  for (i = 0; i < 2*FFT_LEN; i++) {
    fftL[i] = 0.0;
    fftR[i] = 0.0;
    fftLout[i] = 0.0;
    fftRout[i] = 0.0;
  }
}

void convolverInit() {
  unsigned int i;

  convolverClearOutputBuffers();

  for (i = 0; i < FFT_LEN; i++) {
    impulseDataLeft[i] = 0.0;
    impulseDataRight[i] = 0.0;
  }
  for (i = 0; i < 2*FFT_LEN; i++) {
    responseLeft[i] = 0.0;
    responseRight[i] = 0.0;
  }
}

int convolverReadImpulse(char *filename, int autoAdjust, float adjust) {
  unsigned int i, fft_len, effSize, ffts;  
  unsigned int count, impulseSamples;
  double power, powL, powR;
  float buf[FRAMES_PER_BLOCK * SAMPLES_PER_FRAME];
  UINT br;    // bytes read
  FIL Fil;

  if (*filename == 0)
    return(RET_ERR);

  if (f_open(&Fil, filename, FA_READ) != FR_OK)
    return(RET_ERR); // invalid file
  if (f_read(&Fil, (UINT8 *)buf, 88, &br) != FR_OK)
    return(RET_ERR);

  power = powL = powR = 0.0;
  impulseSamples = 0;

  // read the impulse file
  for (;;) {
    if (f_read(&Fil, buf, FRAMES_PER_BLOCK * SAMPLES_PER_FRAME * sizeof(float), &count) != FR_OK)
      break;
    count /= sizeof(float);
    if (count == 0)
      break;  // done

    if (impulseSamples + (count>>1) > IMPULSE_MAX_SIZE)
      count = (IMPULSE_MAX_SIZE - impulseSamples)<<1;  // don't exceed IMPULSE_MAX_SIZE

    for (i = 0; i < count; i += 2) {
      impulseDataLeft[impulseSamples] = buf[i];
      impulseDataRight[impulseSamples] = buf[i+1];
      powL += buf[i]*buf[i];
      powR += buf[i+1]*buf[i+1];
      impulseSamples++;
    }
  }
  f_close(&Fil);
  power = powL > powR ? powL : powR;
  power = sqrt(power);
  ffts = 8;
  while ((ffts < impulseSamples) && (ffts < (1<<20)))
    ffts <<= 1;
  ffts *= 2;
  fft_len = ffts;

  if (fft_len != FFT_LEN)
    return(RET_ERR);
  
  // limit boost to 30dB
  if (autoAdjust) {
    if (power < 0.02)
      scale_coef = 1.0f / ((float)fft_len * 0.02f);
    else 
      scale_coef = 1.0f / ((float)fft_len * (float)power);
  }
  else
    scale_coef = 1.0f / ((float)fft_len);

  scale_coef2 = (float)pow(10.0, ((float)adjust - (float)300.0) / 200.0);

  effSize = fft_len < impulseSamples ? fft_len: impulseSamples;
  
  // do FFT spectrum of impulse  
  for (i = 0; i < effSize; i++) {
    responseLeft[i<<1] = impulseDataLeft[i];
    responseLeft[(i<<1)+1] = 0.0;
    responseRight[i<<1] = impulseDataRight[i];
    responseRight[(i<<1)+1] = 0.0;
  }
  if (effSize < fft_len) {
    for (i = effSize; i < fft_len; i++) {
      responseLeft[i<<1] = 0.0;
      responseLeft[(i<<1)+1] = 0.0;
      responseRight[i<<1] = 0.0;
      responseRight[(i<<1)+1] = 0.0;
    }
  }
  cdft(2*fft_len, 1, responseLeft);
  cdft(2*fft_len, 1, responseRight);

  return(RET_OK);
}

void convolverConvolve(float *bufp, int unbuf) {
  register unsigned int i, j;
  float c1, c2, r1, r2, *buf;

  buf = bufp;
  if (unbuf != 0) {
    for(i = 0; i < FFT_LEN; i++)
      *bufp++ = 0.0;
    bufp = buf;
  }
  // ready fft data
  for (i = 0; i < HALF_FFT_LEN; i++) {
    j = i<<1;
    fftL[j]= (float)*bufp++;
    fftL[j+1] = 0.0;
    fftR[j] = (float)*bufp++;
    fftR[j+1] = 0.0;
  }
  bufp = buf;

  for (i = HALF_FFT_LEN; i < FFT_LEN; i++) {
    j = i<<1;
    fftL[j] = 0.0;
    fftL[j+1] = 0.0;
    fftR[j] = 0.0;
    fftR[j+1] = 0.0;
  }
  // forward fft
  cdft(2*FFT_LEN, 1, fftL);
  cdft(2*FFT_LEN, 1, fftR);

  // impulse multiply
  for (i = 0; i < FFT_LEN; i++) {
    j = i<<1;
    r1 = fftL[j];
    r2 = responseLeft[j];
    c1 = fftL[j+1];
    c2 = responseLeft[j+1];

    fftLout[j] = r1*r2-c1*c2;
    fftLout[j+1] = c1*r2+r1*c2;

    r1 = fftR[j];
    r2 = responseRight[j];
    c1 = fftR[j+1];
    c2 = responseRight[j+1];

    fftRout[j] = r1*r2-c1*c2;
    fftRout[j+1] = c1*r2+r1*c2;
  }
  cdft(2*FFT_LEN, -1, fftLout);
  cdft(2*FFT_LEN, -1, fftRout);

  for (i = 0; i < HALF_FFT_LEN; i++) {
    lobuf[i] += fftLout[i<<1] * scale_coef;
    robuf[i] += fftRout[i<<1] * scale_coef;
  }
  for (i = 0; i < HALF_FFT_LEN; i++) {
    bufp[i<<1] = (lobuf[i] * scale_coef2);
    bufp[(i<<1)+1] = (robuf[i] * scale_coef2);
  }

  // copy to old buffer
  if (unbuf == 0) {
    for (i = 0; i < HALF_FFT_LEN; i++) {
      lobuf[i] = fftLout[(HALF_FFT_LEN+i)<<1] * scale_coef;
      robuf[i] = fftRout[(HALF_FFT_LEN+i)<<1] * scale_coef;
    }
  }
}
