//-----------------------------------------------------------------------------
//
//                   ** WARNING! ** 
//    This file was generated automatically by a tool.
//    Re-running the tool will overwrite this file.
//    You should copy this file to a custom location
//    before adding any customization in the copy to
//    prevent loss of your changes when the tool is
//    re-run.
//
//-----------------------------------------------------------------------------
#include <string.h>

#include <FreeRTOS.h>
#include <task.h>
#include <queue.h>
#include <semphr.h>

#include "defs.h"
#include <math.h>
#include "fifo.h"
#include "cdma.h"
#include "convolver.h"
#include "xover.h"
#include "fatfs.h"
#include "Zynq7000_intc.h"

#include "D:\MicroFrameworkPK_v4_3\DeviceCode\Targets\Native\Zynq7000\DeviceCode\cpu_locks.h"

typedef struct {
  UINT32 Chan1Data;
  UINT32 Chan1Tri;
  UINT32 Chan2Data;
  UINT32 Chan2Tri;
  UINT32 nc[43];
  UINT32 Gier;
  UINT32 Ier;
  UINT32 Isr;
} GPIO_Type;

#define GPIO ((volatile GPIO_Type*)(XPAR_SPDIF_AXI_GPIO_0_BASEADDR))

#define PLAY 0
#define PAUSE 1
#define MUTE 2

static FATFS FatFs;  // FatFs work area needed for each volume
static FIL Fil;      // File object needed for each open file

// global variables (used in fifo.c)
int activeBuf;  // switch between buf0 and buf1
short *shortBuf = (short *)0x02800000;
short *shortBuf0 = (short *)0x02880000;  // size = NB_FRAMES*NB_CHAN*sizeof(short) = 524288
short *shortBuf1 = (short *)0x02900000;
SemaphoreHandle_t fifoSemaphore;
int exitPlay;

// local variables
static float *floatBufConv = (float *)0x02980000;  // size = NB_FRAMES*SAMPLES_PER_FRAME*sizeof(float) = 524288
static float *floatBufXO = (float *)0x02a00000;  // size = NB_FRAMES*NB_CHAN*sizeof(float) = 1048576

static float convolverAdjust;
static float level[NB_SET]; // volume applied to each relevant sample (in dB)

// delays
#define MAX_DELAY 40  // in millisecond
#define DELAY_BUFSIZE  ((MAX_SAMPLE_RATE * MAX_DELAY)/1000)*2
static float delayBuf0[DELAY_BUFSIZE], delayBuf1[DELAY_BUFSIZE];
static unsigned int delayMax;  // highest set delay
static unsigned int delay[NB_SET]; // values returned during calibration
static unsigned int delayIndex[NB_SET];  // current index position

// crossovers
static int cutoff1 = 0;
static int filterOrder1 = 2;  // default 12dB/octave
static int cutoff2 = 0;
static int filterOrder2 = 2;  // default 12dB/octaveS

static unsigned int nbChan = SAMPLES_PER_FRAME; // number of output channels (default = 2)
static unsigned int nbFrames; // number of frames read from file

#define PLAY_TASK_PRIORITY   (tskIDLE_PRIORITY+1)
static xQueueHandle playQueue;

static char filename[100];  // wave filename
static int curFpos;
static SemaphoreHandle_t playSemaphore;
static int fposUpdateIrqnum;

// function prototypes
static void playerTask(void *pvParameters);
static int play();

static void initParams();
static void initLevel();
static void initDelay();
static void xover(float *src, float *dest, int nb_chan, int nb_frames);
static void copyDelayBuf(float *src, float *dest, unsigned int len, unsigned int bufSize);
static float getDelayedValue(int idx);
static void int16_to_float32(short *sourceBuffer, float *targetBuffer, int numSamples);
static void float32_to_int16(float *sourceBuffer, short *targetBuffer, int numSamples);

void interruptSlave(int irqId, int data1, int data2);


// ---------------------------------------------------------------------------
// copy data chunck corresponding to the delay shift (from end of data buffer)
static void copyDelayBuf(float *src, float *dest, unsigned int len, unsigned int bufSize) {
  float *inbufp;

  if (len != 0) {
    if (bufSize > len) {
      inbufp = src + (bufSize - len);
      for (int i = 0; i < len; i++)
        *dest++ = *inbufp++;
    }
    else
      for (int i = 0; i < len; i++)
        *dest++ = 0;
  }
}

// ---------------------------------------------------------------------------
static float getDelayedValue(int idx) {
  float *inbufp;

  if (delayIndex[idx] < delayMax)
    inbufp = (activeBuf == 0 ? delayBuf0 : delayBuf1) + delayIndex[idx];
  else
    inbufp = floatBufConv + (delayIndex[idx]-delayMax);
  
  if (idx % 2 != 0)
    inbufp++; // right channel
  delayIndex[idx] += 2;

  return(*inbufp);
}

// ---------------------------------------------------------------------------
static void int16_to_float32(short *sourceBuffer, float *targetBuffer, int numSamples) {
  for (int i = 0; i < numSamples; i++) {
    float samp = *sourceBuffer * (1.0f / 32768.0f);
    *targetBuffer = samp;
    sourceBuffer++;
    targetBuffer++;
  }
}

// ---------------------------------------------------------------------------
static void float32_to_int16(float *sourceBuffer, short *targetBuffer, int numSamples) {
  for (int i = 0; i < numSamples; i++) {
    short samp = (short) (*sourceBuffer * (32767.0f));
    *targetBuffer = samp;
    sourceBuffer++;
    targetBuffer++;
  }
}

// ---------------------------------------------------------------------------
static void initLevel() {
  int volume, balanceVal, balanceLeft, balanceRight;
  int lev[NB_SET];
 
  volume = 0;
  balanceVal = +1;  // left -3dB
  lev[FPL] = lev[FPR] = lev[LPL] = lev[LPR] = lev[BPL] = lev[BPR] = lev[HPL] = lev[HPR] = 0;
  balanceLeft = (balanceVal < 0 ? balanceVal: 0);
  balanceRight = (balanceVal > 0 ? -balanceVal: 0);

  for (int i = 0; i < NB_SET; i += 2) {
    level[i] = (float)pow(10.0, (float)(volume + balanceLeft + lev[i]) / 20.0);
    level[i+1] = (float)pow(10.0, (float)(volume + balanceRight + lev[i+1]) / 20.0);
  }
}

// ---------------------------------------------------------------------------
static void initDelay() {
  // values set during calibration (latency + sound travel time)
  delay[FPL] = delay[FPR] = delay[LPL] = delay[LPR] = delay[BPL] = delay[BPR] = 0;
  delay[HPL] = 300;
  delay[HPR] = 490;

  // stereo signal so we need to copy 2 times the number of samples to buffer
  for (int i = 0; i < NB_SET; i++)
    delay[i] *= 2;

  // find the highest value
  delayMax = 0;
  for (int i = 0; i < NB_SET; i++)
    if (delayMax < delay[i])
      delayMax = delay[i];
}

// ---------------------------------------------------------------------------
static void initParams() {
  exitPlay = FALSE;
  filename[0] = 0;
  activeBuf = 0;
  convolverAdjust = 300.0;  // = 300 + (x * 10) i.e.: -4.2dB -> 258.0 (0dB -> 300.0)

  cutoff1 = 200;
  cutoff2 = 0;
  filterOrder1 = 4;
  filterOrder2 = 0;

  if (cutoff1)
    nbChan += 2;
  if (cutoff2)
    nbChan += 2;
	
  initLevel();
  initDelay();
}

// ---------------------------------------------------------------------------
int initPlayer() {
  volatile LOCK_Type *lock;
  int err;

  for (;;) {
    err = RET_ERR;
    initParams();

    //CPU_INTC_ActivateInterrupt(SD_IRQ(SD0), SD_ISR(SD0), 0);

    if (CDMA_init(CDMA0) != RET_OK)
      break;

    if (FifoInit(FIFO0) != RET_OK)
      break;

    if (xoverInit(SAMPLING_RATE, cutoff1, cutoff2, filterOrder1, filterOrder2) != RET_OK)
      break;

    convolverInit();  // clear convolver buffers

    for (int i = 0; i < NB_FRAMES*nbChan; i++) {
      shortBuf[i] = 0;
      shortBuf0[i] = 0;
      shortBuf1[i] = 0;
    }
    for (int i = 0; i < DELAY_BUFSIZE; i++) {
      delayBuf0[i] = 0;
      delayBuf1[i] = 0;
    }
    if (f_mount(&FatFs, "", 0) != RET_OK)
    break;

    if (convolverReadImpulse("drc2.wav", 1, convolverAdjust) != RET_OK)
    break;

    if ((fifoSemaphore = xSemaphoreCreateBinary()) == NULL)
      break;

    if ((playSemaphore = xSemaphoreCreateBinary()) == NULL)
      break;

    xSemaphoreGive(playSemaphore);

    playQueue = xQueueCreate(4, 1);
    xTaskCreate(playerTask, (signed char *)"player", configMINIMAL_STACK_SIZE, NULL, PLAY_TASK_PRIORITY, NULL);

    err = RET_OK;
    break;  // exit the forever loop
  }
  return err;
}

// ---------------------------------------------------------------------------
static void xover(float *src, float *dest, int nb_chan, int nb_frames) {
  float sampleLeft, sampleRight;

  for (int i = 0; i < nb_frames; i++) {
    sampleLeft = *src++;
    sampleRight = *src++;

    if (nb_chan == 2) {
      if (delay[FPL] != 0)
        sampleLeft = getDelayedValue(FPL);
      if (delay[FPR] != 0)
        sampleRight = getDelayedValue(FPR);
  
      *dest++ = sampleLeft * level[FPL];
      *dest++ = sampleRight * level[FPR];
    }
    if (nb_chan >= 4) {
      if (delay[HPL] != 0)
        sampleLeft = getDelayedValue(HPL);
      if (delay[HPR] != 0)
        sampleRight = getDelayedValue(HPR);
    
      *dest++ = highpass(LEFT_CHANNEL, sampleLeft) * level[HPL];
      *dest++ = highpass(RIGHT_CHANNEL, sampleRight) * level[HPR];
    }
    if (nb_chan >= 6) {
      if (delay[BPL] != 0)
        sampleLeft = getDelayedValue(BPL);
      if (delay[BPR] != 0)
        sampleRight = getDelayedValue(BPR);

      *dest++ = bandpass(LEFT_CHANNEL, sampleLeft) * level[BPL];
      *dest++ = bandpass(RIGHT_CHANNEL, sampleRight) * level[BPR];
    }
    if (nb_chan > 2) {
      if (delay[LPL] != 0)
        sampleLeft = getDelayedValue(LPL);
      if (delay[LPR] != 0)
        sampleRight = getDelayedValue(LPR);

      *dest++ = lowpass(LEFT_CHANNEL, sampleLeft) * level[LPL];
      *dest++ = lowpass(RIGHT_CHANNEL, sampleRight) * level[LPR];
    }
  }
}

// ---------------------------------------------------------------------------
static int play() {
  UINT br;
  short *shortBuf;
  volatile FIFO_Type *fifo = FIFO_REG(FIFO0);

  // play loop
  int firstLoop = TRUE;
  activeBuf = 0;
  for (;;) {	
    shortBuf = activeBuf == 0 ? shortBuf0 : shortBuf1;

    interruptSlave(fposUpdateIrqnum, curFpos, 0);

    if (f_read(&Fil, (UINT8 *)shortBuf, NB_FRAMES*SAMPLES_PER_FRAME*sizeof(short), &br) != FR_OK)
      break;
    if (br == 0)  // end of file
      break;
    curFpos += br;

    nbFrames = br / (SAMPLES_PER_FRAME*sizeof(short));

    int16_to_float32(shortBuf, floatBufConv, nbFrames*SAMPLES_PER_FRAME);
    convolverConvolve(floatBufConv, 0);

    // copy delay data chunk into opposite delay buffer (i.e. accessed first after next flip)
    copyDelayBuf(floatBufConv, activeBuf == 0 ? delayBuf1 : delayBuf0, delayMax, nbFrames*SAMPLES_PER_FRAME);
    for (int i = 0; i < NB_SET; i++)
      delayIndex[i] = delayMax - delay[i];

    xover(floatBufConv, floatBufXO, nbChan, nbFrames);
    float32_to_int16(floatBufXO, (short *)shortBuf, nbFrames*nbChan);
 
    if (firstLoop == TRUE) {
      firstLoop = FALSE;
      fifo->IER |= XLLF_INT_ALL_MASK;  // enable FIFO interrupt
    }
    xSemaphoreTake(fifoSemaphore, portMAX_DELAY);
    if (exitPlay == TRUE)
      break;

    activeBuf = activeBuf == 0 ? 1 : 0; // flip the switch
  }
  // clear all residual data
  for (int i = 0; i < NB_FRAMES*nbChan; i++) {
    shortBuf0[i] = 0;
    shortBuf1[i] = 0;
  }
  for (int i = 0; i < DELAY_BUFSIZE; i++) {
    delayBuf0[i] = 0;
    delayBuf1[i] = 0;
  }
  convolverClearOutputBuffers();
  xSemaphoreGive(playSemaphore);

  return RET_OK;
}

// ---------------------------------------------------------------------------
static void playerTask(void *pvParameters) {
  uint8 cmd;

  for (;;) {
    xQueueReceive(playQueue, &cmd, portMAX_DELAY);
    exitPlay = FALSE;
    play();
  }
}

// ---------------------------------------------------------------------------
void ToCpu1_setIrq(int fposUpdateIrq) {
  fposUpdateIrqnum = fposUpdateIrq;
}

// ---------------------------------------------------------------------------
void ToCpu1_cdCommand(unsigned char cmd, char *name, int fpos) {
  UINT br;
  int err;
  volatile FIFO_Type *fifo = FIFO_REG(FIFO0);

  err = RET_ERR;
  for (;;) {
    if (name[0] != 0) {
      // new file to play
      GPIO->Chan1Data = MUTE;
      exitPlay = TRUE;  // stop play session
      xSemaphoreTake(playSemaphore, portMAX_DELAY);  // wait for play loop to exit
      if (filename[0] != 0)
        f_close(&Fil);

      strcpy(filename, name);
      if (f_open(&Fil, filename, FA_READ) != FR_OK)
        break;
      if (f_read(&Fil, (UINT8 *)shortBuf0, 68, &br) != FR_OK)  // skip wavefile header
        break;
      curFpos = 68;
      GPIO->Chan1Data = PLAY;  // mute off
    }
    switch (fpos) {
    case 0:
      // pause
      GPIO->Chan1Data = PAUSE;
      fifo->IER &= ~XLLF_INT_ALL_MASK; // disable FIFO interrupt
      break;

    case -1:
      // resume play (after pause)
      GPIO->Chan1Data = PLAY;
      fifo->IER |= XLLF_INT_ALL_MASK;  // enable FIFO interrupt
      break;

    default:
      curFpos = fpos;
      if (f_lseek(&Fil, curFpos) == FR_OK) {
        // tell playTask to start the play loop
        xQueueSend(playQueue, &cmd, 0);
      }
      break;
    }
    err = RET_OK;
    break;  // exit the forever loop
  }
  // end pf functionSlave call
}

