﻿using System;
using System.IO;
using Microsoft.SPOT;
using Microsoft.SPOT.Input;
using Microsoft.SPOT.Presentation;
using Microsoft.SPOT.Presentation.Controls;
using Microsoft.SPOT.Presentation.Media;
using Microsoft.SPOT.Touch;
using Microsoft.SPOT.IO;

namespace CDplayer
{
    public class Program : Microsoft.SPOT.Application
    {
        Window mainWindow;
        Canvas canvas;
        CDS cdfolder, cdlist;
        CD cdplaying;
       
        public static void Main()
        {
            Program cdPlayer = new Program();
            Touch.Initialize(cdPlayer);
            Window mainWindow = cdPlayer.CreateWindow();
            cdPlayer.Run(mainWindow);
        }

        public Window CreateWindow()
        {
            string rootDirectory = VolumeInfo.GetVolumes()[0].RootDirectory;

            mainWindow = new Window();
            mainWindow.Height = SystemMetrics.ScreenHeight;
            mainWindow.Width = SystemMetrics.ScreenWidth;
            mainWindow.Background = new SolidColorBrush(ColorUtility.ColorFromRGB(247, 242, 154));  // colorChampagne

            canvas = new Canvas();

            // folders of cd's
            cdfolder = new CDS(CDS_TYPE.FOLDER, canvas, 3, 3, 83, 78);  // displays icons on 3 rows & 3 columns, icon size
            cdfolder.Init(rootDirectory + "\\root");
            cdfolder.cdList += new CDS.cdListEvent(selectedList); // clicking on any folder icon goes to corresponding cdlist screen
            cdfolder.cdPlaying += new CDS.cdPlayingEvent(playingCd); // clicking on playing icon goes to cdplaying screen

            // list of cd's per folder
            cdlist = new CDS(CDS_TYPE.LIST, canvas, 4, 4, 62, 58);  // displays icons on 4 rows & 4 columns, icon size
            cdlist.cdFolder += new CDS.cdFolderEvent(selectedFolder); // clicking on first icon goes to cdfolder screen
            cdlist.cdList += new CDS.cdListEvent(playingCd); // clicking on any cd icon goes to cdplaying screen
            cdlist.cdPlaying += new CDS.cdPlayingEvent(playingCd); // clicking on playing icon goes to cdplaying screen

            // selected cd
            cdplaying = new CD(canvas);
            cdplaying.RootDirectory = rootDirectory;
            cdplaying.cd += new CD.cdEvent(selectedList);  // clicking on mdir icon returns to previous screen (cdfolder or cdlist)

            cdfolder.Display();  // starts with cdfolder screen
            mainWindow.Child = canvas;
            mainWindow.Visibility = Visibility.Visible;
            return mainWindow;
        }

        void selectedFolder(object sender)
        {
            cdfolder.Display();
        }

        void selectedList(object sender, CDS_TYPE id, string dir)
        {
            cdlist.Init(dir);
            cdlist.Display();
        }

        void selectedList(object sender, CDS_TYPE cdsType)
        {
            cdfolder.PlayingCD = cdplaying.PlayingCD;
            cdlist.PlayingCD = cdplaying.PlayingCD;
            if (cdsType == CDS_TYPE.FOLDER)
                cdfolder.Display();
            else
                cdlist.Display();
        }

        void playingCd(object sender, CDS_TYPE cdsType, string filename)
        {
            cdplaying.Display(cdsType, filename);
        }
    }
}
