﻿using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Input;
using Microsoft.SPOT.Presentation.Controls;
using Microsoft.SPOT.Presentation.Media;

namespace CDplayer
{
    public class Slider
    {
        public delegate void sliderEvent(object sender, bool dir);
        public event sliderEvent Slide;

        Canvas canvas;
        Bitmap bm;
        Image img;
        int left, top, width, length, cursorHeight;
        Color backgroundColor, cursorColor;
        int cursorY = 2;

        public Slider(Canvas canvas, int left, int top, int width, int length, int cursorHeight, Color backgroundColor, Color cursorColor)
        {
            this.canvas = canvas;
            this.left = left;
            this.top = top;
            this.width = width;
            this.length = length;
            this.cursorHeight = cursorHeight;
            this.backgroundColor = backgroundColor;
            this.cursorColor = cursorColor;
            bm = new Bitmap(width, length);
            img = new Image(bm);
            img.TouchDown += slider_TouchMove;
            img.TouchMove += slider_TouchMove;
        }

        public void Display()
        {
            // frame
            bm.DrawRectangle(Color.Black, 1, 0, 0, width, length, 0, 0, backgroundColor, 0, 0, backgroundColor, 0, 0, Bitmap.OpacityOpaque);
            // cursor
            bm.DrawRectangle(cursorColor, 0, 2, cursorY, width - 4, cursorHeight, 0, 0, cursorColor, 0, 0, cursorColor, 0, 0, Bitmap.OpacityOpaque);
            canvas.Children.Add(img);
            Canvas.SetLeft(img, left);
            Canvas.SetTop(img, top);
        }

        private void slider_TouchMove(object sender, TouchEventArgs e)
        {
            int y = e.Touches[0].Y - top;
            if (y < cursorY || y > cursorY + cursorHeight)  // ignore if touch on the cursor itself
            {
                bool moveDown = false;  // above the cursor 
                if (y > cursorY + cursorHeight)
                    moveDown = true;    // below the cursor
                cursorY += moveDown ? cursorHeight : -cursorHeight;

                if (cursorY < 2)    // min
                    cursorY = 2;
                else if (cursorY + cursorHeight > length)   // max
                    cursorY = length - cursorHeight;
                else
                    Slide(sender, moveDown);
            }
        }
    }
}
